"""Gene fixtures for testing."""
from geneweaver.core.schema.gene import Gene, GeneDatabase, GeneDatabaseRow, GeneRow

GENE_ROWS = [
    GeneRow(
        ode_gene_id=606485,
        ode_ref_id="100683049",
        gdb_id=1,
        sp_id=11,
        ode_pref=False,
        ode_date="2020-05-05",
        old_ode_gene_ids=None,
    ),
    GeneRow(
        ode_gene_id=606486,
        ode_ref_id="100683125",
        gdb_id=1,
        sp_id=11,
        ode_pref=False,
        ode_date="2020-05-05",
        old_ode_gene_ids=None,
    ),
    GeneRow(
        ode_gene_id=606487,
        ode_ref_id="100683204",
        gdb_id=1,
        sp_id=11,
        ode_pref=False,
        ode_date="2020-05-05",
        old_ode_gene_ids=None,
    ),
    GeneRow(
        ode_gene_id=606488,
        ode_ref_id="100683346",
        gdb_id=1,
        sp_id=11,
        ode_pref=False,
        ode_date="2020-05-05",
        old_ode_gene_ids=None,
    ),
    GeneRow(
        ode_gene_id=606489,
        ode_ref_id="100856548",
        gdb_id=1,
        sp_id=11,
        ode_pref=False,
        ode_date="2020-05-05",
        old_ode_gene_ids=None,
    ),
    GeneRow(
        ode_gene_id=606490,
        ode_ref_id="111094793",
        gdb_id=1,
        sp_id=11,
        ode_pref=False,
        ode_date="2020-05-05",
        old_ode_gene_ids=None,
    ),
    GeneRow(
        ode_gene_id=606491,
        ode_ref_id="111094899",
        gdb_id=1,
        sp_id=11,
        ode_pref=False,
        ode_date="2020-05-05",
        old_ode_gene_ids=None,
    ),
    GeneRow(
        ode_gene_id=606492,
        ode_ref_id="111094906",
        gdb_id=1,
        sp_id=11,
        ode_pref=False,
        ode_date="2020-05-05",
        old_ode_gene_ids=None,
    ),
    GeneRow(
        ode_gene_id=606493,
        ode_ref_id="111094786",
        gdb_id=1,
        sp_id=11,
        ode_pref=False,
        ode_date="2020-05-05",
        old_ode_gene_ids=None,
    ),
    GeneRow(
        ode_gene_id=606494,
        ode_ref_id="612605",
        gdb_id=1,
        sp_id=11,
        ode_pref=False,
        ode_date="2020-05-05",
        old_ode_gene_ids=None,
    ),
    GeneRow(
        ode_gene_id=92095,
        ode_ref_id="A1BG",
        gdb_id=7,
        sp_id=2,
        ode_pref=True,
        ode_date="2020-05-05",
        old_ode_gene_ids=None,
    ),
    GeneRow(
        ode_gene_id=92085,
        ode_ref_id="A1BG-AS1",
        gdb_id=7,
        sp_id=2,
        ode_pref=True,
        ode_date="2020-05-05",
        old_ode_gene_ids=None,
    ),
    GeneRow(
        ode_gene_id=75095,
        ode_ref_id="A1CF",
        gdb_id=7,
        sp_id=2,
        ode_pref=True,
        ode_date="2020-05-05",
        old_ode_gene_ids=None,
    ),
    GeneRow(
        ode_gene_id=74678,
        ode_ref_id="A2M",
        gdb_id=7,
        sp_id=2,
        ode_pref=True,
        ode_date="2020-05-05",
        old_ode_gene_ids=None,
    ),
    GeneRow(
        ode_gene_id=157301,
        ode_ref_id="7SK",
        gdb_id=7,
        sp_id=3,
        ode_pref=True,
        ode_date="2020-05-05",
        old_ode_gene_ids=None,
    ),
    GeneRow(
        ode_gene_id=154481,
        ode_ref_id="7SK",
        gdb_id=7,
        sp_id=3,
        ode_pref=True,
        ode_date="2020-05-05",
        old_ode_gene_ids=None,
    ),
    GeneRow(
        ode_gene_id=153231,
        ode_ref_id="7SK",
        gdb_id=7,
        sp_id=3,
        ode_pref=True,
        ode_date="2020-05-05",
        old_ode_gene_ids=None,
    ),
    GeneRow(
        ode_gene_id=151248,
        ode_ref_id="7SK",
        gdb_id=7,
        sp_id=3,
        ode_pref=True,
        ode_date="2020-05-05",
        old_ode_gene_ids=None,
    ),
    GeneRow(
        ode_gene_id=191467,
        ode_ref_id="a008504",
        gdb_id=7,
        sp_id=4,
        ode_pref=True,
        ode_date="2020-05-05",
        old_ode_gene_ids=None,
    ),
    GeneRow(
        ode_gene_id=193962,
        ode_ref_id="a0264234",
        gbd_id=7,
        sp_id=4,
        ode_pref=True,
        ode_date="2020-05-05",
        old_ode_gene_ids=None,
    ),
    GeneRow(
        ode_gene_id=190256,
        ode_ref_id="a0264239",
        gbd_id=7,
        sp_id=4,
        ode_pref=True,
        ode_date="2020-05-05",
        old_ode_gene_ids=None,
    ),
    GeneRow(
        ode_gene_id=194073,
        ode_ref_id="a0274750",
        gbd_id=7,
        sp_id=4,
        ode_pref=True,
        ode_date="2020-05-05",
        old_ode_gene_ids=None,
    ),
    GeneRow(
        ode_gene_id=290564,
        ode_ref_id="7SLRNA:CR32864",
        gbd_id=7,
        sp_id=5,
        ode_pref=True,
        ode_date="2020-05-05",
        old_ode_gene_ids=None,
    ),
    GeneRow(
        ode_gene_id=290563,
        ode_ref_id="7SLRNA:CR42652",
        gbd_id=7,
        sp_id=5,
        ode_pref=True,
        ode_date="2020-05-05",
        old_ode_gene_ids=None,
    ),
    GeneRow(
        ode_gene_id=461964,
        ode_ref_id="825-Oak",
        gbd_id=7,
        sp_id=5,
        ode_pref=True,
        ode_date="2020-05-05",
        old_ode_gene_ids=None,
    ),
    GeneRow(
        ode_gene_id=290329,
        ode_ref_id="82Y",
        gbd_id=7,
        sp_id=5,
        ode_pref=True,
        ode_date="2020-05-05",
        old_ode_gene_ids=None,
    ),
]

GENES = [
    Gene(
        id=606485,
        reference_id="100683049",
        gene_database="Entrez",
        species="Mus musculus",
        preferred=False,
        date="2020-05-05",
    ),
    Gene(
        id=606486,
        reference_id="100683125",
        gene_database="Entrez",
        species="Mus musculus",
        preferred=False,
        date="2020-05-05",
    ),
    Gene(
        id=606487,
        reference_id="100683204",
        gene_database="Entrez",
        species="Mus musculus",
        preferred=False,
        date="2020-05-05",
    ),
    Gene(
        id=606488,
        reference_id="100683346",
        gene_database="Entrez",
        species="Mus musculus",
        preferred=False,
        date="2020-05-05",
    ),
    Gene(
        id=606489,
        reference_id="100856548",
        gene_database="Entrez",
        species="Mus musculus",
        preferred=False,
        date="2020-05-05",
    ),
    Gene(
        id=606490,
        reference_id="111094793",
        gene_database="Entrez",
        species="Mus musculus",
        preferred=False,
        date="2020-05-05",
    ),
    Gene(
        id=606491,
        reference_id="111094899",
        gene_database="Entrez",
        species="Mus musculus",
        preferred=False,
        date="2020-05-05",
    ),
    Gene(
        id=606492,
        reference_id="111094906",
        gene_database="Entrez",
        species="Mus musculus",
        preferred=False,
        date="2020-05-05",
    ),
    Gene(
        id=606493,
        reference_id="111094786",
        gene_database="Entrez",
        species="Mus musculus",
        preferred=False,
        date="2020-05-05",
    ),
    Gene(
        id=606494,
        reference_id="612605",
        gene_database="Entrez",
        species="Mus musculus",
        preferred=False,
        date="2020-05-05",
    ),
    Gene(
        id=92095,
        reference_id="A1BG",
        gene_database="Gene Symbol",
        species="Homo sapiens",
        preferred=True,
        date="2020-05-05",
    ),
    Gene(
        id=92085,
        reference_id="A1BG-AS1",
        gene_database="Gene Symbol",
        species="Homo sapiens",
        preferred=True,
        date="2020-05-05",
    ),
    Gene(
        id=75095,
        reference_id="A1CF",
        gene_database="Gene Symbol",
        species="Homo sapiens",
        preferred=True,
        date="2020-05-05",
    ),
    Gene(
        id=74678,
        reference_id="A2M",
        gene_database="Gene Symbol",
        species="Homo sapiens",
        preferred=True,
        date="2020-05-05",
    ),
    Gene(
        id=157301,
        reference_id="7SK",
        gene_database="Gene Symbol",
        species="Rattus norvegicus",
        preferred=True,
        date="2020-05-05",
    ),
    Gene(
        id=154481,
        reference_id="7SK",
        gene_database="Gene Symbol",
        species="Rattus norvegicus",
        preferred=True,
        date="2020-05-05",
    ),
    Gene(
        id=153231,
        reference_id="7SK",
        gene_database="Gene Symbol",
        species="Rattus norvegicus",
        preferred=True,
        date="2020-05-05",
    ),
    Gene(
        id=151248,
        reference_id="7SK",
        gene_database="Gene Symbol",
        species="Rattus norvegicus",
        preferred=True,
        date="2020-05-05",
    ),
    Gene(
        id=191467,
        reference_id="a008504",
        gene_database="Gene Symbol",
        species="Danio rerio",
        preferred=True,
        date="2020-05-05",
    ),
    Gene(
        id=193962,
        reference_id="a0264234",
        gene_database="Gene Symbol",
        species="Danio rerio",
        preferred=True,
        date="2020-05-05",
    ),
    Gene(
        id=190256,
        reference_id="a0264239",
        gene_database="Gene Symbol",
        species="Danio rerio",
        preferred=True,
        date="2020-05-05",
    ),
    Gene(
        id=194073,
        reference_id="a0274750",
        gene_database="Gene Symbol",
        species="Danio rerio",
        preferred=True,
        date="2020-05-05",
    ),
    Gene(
        id=290564,
        reference_id="7SLRNA:CR32864",
        gene_database="Gene Symbol",
        species="Drosophila melanogaster",
        preferred=True,
        date="2020-05-05",
    ),
    Gene(
        id=290563,
        reference_id="7SLRNA:CR42652",
        gene_database="Gene Symbol",
        species="Drosophila melanogaster",
        preferred=True,
        date="2020-05-05",
    ),
    Gene(
        id=461964,
        reference_id="825-Oak",
        gene_database="Gene Symbol",
        species="Drosophila melanogaster",
        preferred=True,
        date="2020-05-05",
    ),
    Gene(
        id=290329,
        reference_id="82Y",
        gene_database="Gene Symbol",
        species="Drosophila melanogaster",
        preferred=True,
        date="2020-05-05",
    ),
]

GENE_DATABASE_ROWS = [
    GeneDatabaseRow(
        gdb_id=8,
        gdb_name="Unannotated",
        sp_id=0,
        gdb_shortname="unannotated",
        gdb_date="2011-03-14 14:17:13.49449",
        gdb_precision=4,
        gdb_linkout_url=None,
    ),
    GeneDatabaseRow(
        gdb_id=4,
        gdb_name="Ensembl Transcript",
        sp_id=0,
        gdb_shortname="ensembl transcript",
        gdb_date="2011-03-14 14:17:13.49449",
        gdb_precision=2,
        gdb_linkout_url=None,
    ),
    GeneDatabaseRow(
        gdb_id=3,
        gdb_name="Ensembl Protein",
        sp_id=0,
        gdb_shortname="ensembl protein",
        gdb_date="2011-03-14 14:17:13.49449",
        gdb_precision=2,
        gdb_linkout_url=None,
    ),
    GeneDatabaseRow(
        gdb_id=7,
        gdb_name="Gene Symbol",
        sp_id=0,
        gdb_shortname="symbol",
        gdb_date="2011-03-14 14:17:13.49449",
        gdb_precision=4,
        gdb_linkout_url=None,
    ),
    GeneDatabaseRow(
        gdb_id=10,
        gdb_name="MGI",
        sp_id=1,
        gdb_shortname="mgi",
        gdb_date="2011-03-14 14:17:13.49449",
        gdb_precision=1,
        gdb_linkout_url=None,
    ),
    GeneDatabaseRow(
        gdb_id=11,
        gdb_name="HGNC",
        sp_id=2,
        gdb_shortname="hgnc",
        gdb_date="2011-03-14 14:17:13.49449",
        gdb_precision=1,
        gdb_linkout_url=None,
    ),
    GeneDatabaseRow(
        gdb_id=12,
        gdb_name="RGD",
        sp_id=3,
        gdb_shortname="rgd",
        gdb_date="2011-03-14 14:17:13.49449",
        gdb_precision=1,
        gdb_linkout_url=None,
    ),
    GeneDatabaseRow(
        gdb_id=13,
        gdb_name="ZFIN",
        sp_id=4,
        gdb_shortname="zfin",
        gdb_date="2011-03-14 14:17:13.49449",
        gdb_precision=1,
        gdb_linkout_url=None,
    ),
    GeneDatabaseRow(
        gdb_id=14,
        gdb_name="FlyBase",
        sp_id=5,
        gdb_shortname="flybase",
        gdb_date="2011-03-14 14:17:13.49449",
        gdb_precision=1,
        gdb_linkout_url=None,
    ),
    GeneDatabaseRow(
        gdb_id=1,
        gdb_name="Entrez",
        sp_id=0,
        gdb_shortname="entrez",
        gdb_date="2011-03-14 14:17:13.49449",
        gdb_precision=2,
        gdb_linkout_url=None,
    ),
    GeneDatabaseRow(
        gdb_id=2,
        gdb_name="Ensembl Gene",
        sp_id=0,
        gdb_shortname="ensembl",
        gdb_date="2011-03-14 14:17:13.49449",
        gdb_precision=2,
        gdb_linkout_url=None,
    ),
    GeneDatabaseRow(
        gdb_id=5,
        gdb_name="Unigene",
        sp_id=0,
        gdb_shortname="unigene",
        gdb_date="2011-03-14 14:17:13.49449",
        gdb_precision=3,
        gdb_linkout_url=None,
    ),
    GeneDatabaseRow(
        gdb_id=15,
        gdb_name="Wormbase",
        sp_id=8,
        gdb_shortname="wormbase",
        gdb_date="2011-03-14 14:17:13.49449",
        gdb_precision=1,
        gdb_linkout_url=None,
    ),
    GeneDatabaseRow(
        gdb_id=16,
        gdb_name="SGD",
        sp_id=9,
        gdb_shortname="sgd",
        gdb_date="2011-03-14 14:17:13.49449",
        gdb_precision=1,
        gdb_linkout_url=None,
    ),
    GeneDatabaseRow(
        gdb_id=17,
        gdb_name="miRBase",
        sp_id=0,
        gdb_shortname="mirbase",
        gdb_date="2011-09-12 14:38:49.943681",
        gdb_precision=2,
        gdb_linkout_url=None,
    ),
    GeneDatabaseRow(
        gdb_id=20,
        gdb_name="CGNC",
        sp_id=10,
        gdb_shortname="cgnc",
        gdb_date="2015-06-05 18:05:13.012876",
        gdb_precision=1,
        gdb_linkout_url=None,
    ),
]

GENE_DATABASES = [
    GeneDatabase(
        name="Unannotated",
        shortname="unannotated",
        species="all",
        data="2011-03-14 14:17:13.49449",
        precision=4,
    ),
    GeneDatabase(
        name="Ensembl Transcript",
        shortname="ensembl transcript",
        species="all",
        data="2011-03-14 14:17:13.49449",
        precision=2,
    ),
    GeneDatabase(
        name="Ensembl Protein",
        shortname="ensembl protein",
        species="all",
        data="2011-03-14 14:17:13.49449",
        precision=2,
    ),
    GeneDatabase(
        name="Gene Symbol",
        shortname="symbol",
        species="all",
        data="2011-03-14 14:17:13.49449",
        precision=4,
    ),
    GeneDatabase(
        name="MGI",
        shortname="mgi",
        species="Mus musculus",
        data="2011-03-14 14:17:13.49449",
        precision=1,
    ),
    GeneDatabase(
        name="HGNC",
        shortname="hgnc",
        species="Homo sapiens",
        data="2011-03-14 14:17:13.49449",
        precision=1,
    ),
    GeneDatabase(
        name="RGD",
        shortname="rgd",
        species="Rattus norvegicus",
        data="2011-03-14 14:17:13.49449",
        precision=1,
    ),
    GeneDatabase(
        name="ZFIN",
        shortname="zfin",
        species="Danio rerio",
        data="2011-03-14 14:17:13.49449",
        precision=1,
    ),
    GeneDatabase(
        name="FlyBase",
        shortname="flybase",
        species="Drosophila melanogaster",
        data="2011-03-14 14:17:13.49449",
        precision=1,
    ),
    GeneDatabase(
        name="Entrez",
        shortname="entrez",
        species="all",
        data="2011-03-14 14:17:13.49449",
        precision=2,
    ),
    GeneDatabase(
        name="Ensembl Gene",
        shortname="ensembl",
        species="all",
        data="2011-03-14 14:17:13.49449",
        precision=2,
    ),
    GeneDatabase(
        name="Unigene",
        shortname="unigene",
        species="all",
        data="2011-03-14 14:17:13.49449",
        precision=3,
    ),
    GeneDatabase(
        name="Wormbase",
        shortname="wormbase",
        species="Caenorhabditis elegans",
        data="2011-03-14 14:17:13.49449",
        precision=1,
    ),
    GeneDatabase(
        name="SGD",
        shortname="sgd",
        species="Saccharomyces cerevisiae",
        data="2011-03-14 14:17:13.49449",
        precision=1,
    ),
    GeneDatabase(
        name="miRBase",
        shortname="mirbase",
        species="all",
        data="2011-09-12 14:38:49.943681",
        precision=2,
    ),
    GeneDatabase(
        name="CGNC",
        shortname="cgnc",
        species="Gallus gallus",
        data="2015-06-05 18:05:13.012876",
        precision=1,
    ),
]
