"""Species fixtures for testing."""
from geneweaver.core.schema.species import Species, SpeciesRow

SPECIES = [
    Species(id=0, name="all", taxonomic_id=0),
    Species(id=1, name="Mus musculus", taxonomic_id=10090),
    Species(id=2, name="Homo sapiens", taxonomic_id=9606),
    Species(id=3, name="Rattus norvegicus", taxonomic_id=10116),
    Species(id=4, name="Danio rerio", taxonomic_id=7955),
    Species(id=5, name="Drosophila melanogaster", taxonomic_id=7227),
    Species(id=6, name="Macaca mulatta", taxonomic_id=9544),
    Species(id=8, name="Caenorhabditis elegans", taxonomic_id=6239),
    Species(id=9, name="Saccharomyces cerevisiae", taxonomic_id=559292),
    Species(id=10, name="Gallus gallus", taxonomic_id=9031),
    Species(id=11, name="Canis familiaris", taxonomic_id=9615),
]


SPECIES_ROWS = [
    SpeciesRow(sp_id=0, sp_name="", sp_tax_id=0, sp_date="2011-03-14"),
    SpeciesRow(
        sp_id=1,
        sp_name="Mus musculus",
        sp_tax_id=10090,
        sp_ref_gdb_id=10,
        sp_date="2011-03-14",
        sp_biomart_info="mmusculus_gene_ensembl  mgi_id",
        sp_source_data={},
    ),
    SpeciesRow(
        sp_id=2,
        sp_name="Homo sapiens",
        sp_tax_id=9606,
        sp_ref_gdb_id=11,
        sp_date="2011-03-14",
        sp_biomart_info="hsapiens_gene_ensembl   hgnc_id",
        sp_source_data={},
    ),
    SpeciesRow(
        sp_id=3,
        sp_name="Rattus norvegicus",
        sp_tax_id=10116,
        sp_ref_gdb_id=12,
        sp_date="2011-03-14",
        sp_biomart_info="rnorvegicus_gene_ensembl        rgd_id",
        sp_source_data={},
    ),
    SpeciesRow(
        sp_id=4,
        sp_name="Danio rerio",
        sp_tax_id=7955,
        sp_ref_gdb_id=13,
        sp_date="2011-03-14",
        sp_biomart_info="drerio_gene_ensembl     zfin_id_id",
        sp_source_data={},
    ),
    SpeciesRow(
        sp_id=5,
        sp_name="Drosophila melanogaster",
        sp_tax_id=7227,
        sp_ref_gdb_id=14,
        sp_date="2011-03-14",
        sp_biomart_info="dmelanogaster_gene_ensembl      flybase_gene_id",
        sp_source_data={},
    ),
    SpeciesRow(
        sp_id=6,
        sp_name="Macaca mulatta",
        sp_tax_id=9544,
        sp_ref_gdb_id=1,
        sp_date="2011-03-14",
        sp_biomart_info="mmulatta_gene_ensembl   ensembl_gene_id",
        sp_source_data={},
    ),
    SpeciesRow(
        sp_id=8,
        sp_name="Caenorhabditis elegans",
        sp_tax_id=6239,
        sp_ref_gdb_id=15,
        sp_date="2011-03-14",
        sp_biomart_info="celegans_gene_ensembl   wormbase_gene",
        sp_source_data={},
    ),
    SpeciesRow(
        sp_id=9,
        sp_name="Saccharomyces cerevisiae",
        sp_tax_id=559292,
        sp_ref_gdb_id=16,
        sp_date="2011-03-14",
        sp_biomart_info="scerevisiae_gene_ensembl        entrezgene_id",
        sp_source_data={},
    ),
    SpeciesRow(
        sp_id=10,
        sp_name="Gallus gallus",
        sp_tax_id=9031,
        sp_ref_gdb_id=20,
        sp_date="2011-03-14",
        sp_biomart_info="ggallus_gene_ensembl    entrezgene_id",
        sp_source_data={},
    ),
    SpeciesRow(
        sp_id=11,
        sp_name="Canis familiaris",
        sp_tax_id=9615,
        sp_ref_gdb_id=1,
        sp_date="2011-03-14",
        sp_biomart_info="cfamiliaris_gene_ensembl        entrezgene_id",
        sp_source_data={},
    ),
]
